import * as globals from '/base-standard/maps/map-globals.js';
import * as utilities from '/base-standard/maps/map-utilities.js';

export function createRandomBorder(iX, iHeight, maxOffset, length) {

    let borderOffsets = new Array(iHeight);
    let terms = new Array(5);

    let baseFreq = 2.0 * Math.PI / length
    for (let i = 1; i <= 5; i++) {
        let k = 1 + TerrainBuilder.getRandomNumber(4, "Random freq")
        let freq = baseFreq * k;
        let phase = random01() * Math.PI * 2;
        let amp = (random01() * (1.2 - 0.8) + 0.8) / i;
        terms[i - 1] = {
            freq: freq,
            phase: phase,
            amp: amp
        };
    }

    for (let h = 0; h < iHeight; ++h) {
        let heightFac = h * (length / iHeight);
        let curValue = 0;
        for (let i = 0; i < 5; i++) {
            curValue += terms[i].amp * Math.sin(terms[i].freq * heightFac + terms[i].phase);
        }
        curValue *= maxOffset * 0.6;
        curValue += (h % 2) == 0 ? 0 : -0.5;
        borderOffsets[h] = Math.round(curValue);
    }

    var ret = {
        base: iX,
        offsets: borderOffsets
    };

    return ret;
}

export function random01() {
    return TerrainBuilder.getRandomNumber(10000, "Random 01") / 10000.0;
}

export function getBorderDistanceX(border, iX, iY, iWidth) {

    let iXmod = (iX + iWidth) % iWidth;
    let borderXmod = (border.base + border.offsets[iY] + iWidth) % iWidth;
    return Math.min(Math.abs(iXmod - borderXmod), iWidth - Math.abs(iXmod - borderXmod))
}

export function getBorderDistanceY(border, iX, iY) {

    let borderYmod = border.base + border.offsets[iX];
    return Math.abs(borderYmod - iY);
}

export function getContinentFromBorders(borderStart, borderMiddle, iX, iY, iWidth) {

    let iXmod = (iX + iWidth) % iWidth;
    let borderStartXmod = (borderStart.base + borderStart.offsets[iY] + iWidth) % iWidth;
    let borderMiddleXmod = (borderMiddle.base + borderMiddle.offsets[iY] + iWidth) % iWidth;

    if (borderStartXmod <= borderMiddleXmod) {
        return iXmod >= borderStartXmod && iXmod <= borderMiddleXmod;
    } else {
        return iXmod >= borderStartXmod || iXmod <= borderMiddleXmod;
    }
}